@@ -31,6 +31,7 @@
 #include "ephy-embed-shell.h"
 #include "ephy-gui.h"
 #include "ephy-lib-type-builtins.h"
+#include "ephy-settings.h"
 #include "ephy-signal-accumulator.h"
 #include "ephy-suggestion.h"
 #include "ephy-title-widget.h"
@@ -1013,6 +1014,12 @@ ephy_location_entry_construct_contents (EphyLocationEntry *entry)
   context = gtk_widget_get_style_context (entry->bookmark);
   gtk_style_context_add_class (context, "entry_icon");
 
+  g_settings_bind (EPHY_SETTINGS_LOCKDOWN,
+                   EPHY_PREFS_LOCKDOWN_BOOKMARK_EDITING,
+                   entry->bookmark_event_box,
+                   "visible",
+                   G_SETTINGS_BIND_GET | G_SETTINGS_BIND_INVERT_BOOLEAN);
+
   /* Reader Mode */
   entry->reader_mode_event_box = gtk_event_box_new ();
   gtk_widget_set_tooltip_text (entry->reader_mode_event_box, _("Toggle reader mode"));
@@ -1453,3 +1460,13 @@ ephy_location_entry_set_progress (EphyLocationEntry *entry,
   entry->progress_fraction = fraction;
   ephy_location_entry_set_fraction_internal (entry);
 }
+
+void
+ephy_location_entry_set_mobile_popdown (EphyLocationEntry *entry,
+                                        gboolean           mobile_popdown)
+{
+  if (mobile_popdown)
+    dzl_suggestion_entry_set_position_func (DZL_SUGGESTION_ENTRY (entry->url_entry), dzl_suggestion_entry_window_position_func, NULL, NULL);
+  else
+    dzl_suggestion_entry_set_position_func (DZL_SUGGESTION_ENTRY (entry->url_entry), position_func, NULL, NULL);
+}
